@@ -26,7 +26,7 @@ from elementpath.datatypes import DateTime, DateTime10, Date, Date10, Time, \
     ArithmeticProxy, Id, is_idrefs, Notation, QName, Base64Binary, HexBinary, \
     NormalizedString, XsdToken, Language, Float, Float10, Integer, AnyURI, \
     BooleanProxy, DecimalProxy, DoubleProxy10, DoubleProxy, StringProxy
-from elementpath.datatypes.atomic_types import AtomicTypeABCMeta
+from elementpath.datatypes.atomic_types import AtomicTypeMeta
 from elementpath.datatypes.helpers import MONTH_DAYS, MONTH_DAYS_LEAP, \
     days_from_common_era, months2days, round_number
 from elementpath.datatypes.datetime import OrderedDateTime
@@ -37,11 +37,11 @@ class AnyAtomicTypeTest(unittest.TestCase):
     def test_invalid_type_name(self):
 
         with self.assertRaises(TypeError):
-            class InvalidAtomicType(metaclass=AtomicTypeABCMeta):
+            class InvalidAtomicType(metaclass=AtomicTypeMeta):
                 name = b'invalid'
 
     def test_validation(self):
-        class AnotherAtomicType(metaclass=AtomicTypeABCMeta):
+        class AnotherAtomicType(metaclass=AtomicTypeMeta):
             pass
 
         self.assertIsNone(AnotherAtomicType.validate(AnotherAtomicType()))
@@ -340,14 +340,30 @@ class DateTimeTypesTest(unittest.TestCase):
         self.assertIsInstance(dt, DateTime)
         self.assertEqual(dt._dt, datetime.datetime(2000, 10, 7, microsecond=100000))
 
-        with self.assertRaises(OverflowError) as ctx:
-            DateTime.fromstring('2000-10-07T00:00:00.1000000')
-        self.assertEqual("Invalid value 1000000 for microsecond", str(ctx.exception))
-
+    def test_issue_36_fromstring_with_more_microseconds_digits(self):
         dt = DateTime.fromstring('2000-10-07T00:00:00.00090001')
         self.assertIsInstance(dt, DateTime)
         self.assertEqual(dt._dt, datetime.datetime(2000, 10, 7, microsecond=900))
 
+        dt = DateTime.fromstring('2000-10-07T00:00:00.0009009999')
+        self.assertIsInstance(dt, DateTime)
+        self.assertEqual(dt._dt, datetime.datetime(2000, 10, 7, microsecond=900))
+
+        dt = DateTime.fromstring('2000-10-07T00:00:00.1000000')
+        self.assertIsInstance(dt, DateTime)
+        self.assertEqual(dt._dt, datetime.datetime(2000, 10, 7, microsecond=100000))
+
+        # Regression test of issue #36
+        tz = Timezone.fromstring('+01:00')
+        dt = DateTime.fromstring('2021-02-21T21:43:03.1121296+01:00')
+        self.assertIsInstance(dt, DateTime)
+        self.assertEqual(dt._dt, datetime.datetime(2021, 2, 21, 21, 43, 3, 112129, tz))
+
+        # From W3C's XQuery/XPath tests
+        dt = DateTime.fromstring('9999-12-31T23:59:59.9999999')
+        self.assertIsInstance(dt, DateTime)
+        self.assertEqual(dt._dt, datetime.datetime(9999, 12, 31, 23, 59, 59, 999999))
+
     def test_date_fromstring(self):
         self.assertIsInstance(Date.fromstring('2000-10-07'), Date)
         self.assertIsInstance(Date.fromstring('-2000-10-07'), Date)
