@@ -229,6 +229,8 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_tree(' (: initial comment :)/ (:2nd comment:)A/B1(: 3rd comment :)/ \n'
                         'C1 (: last comment :)\t', '(/ (/ (/ (A)) (B1)) (C1))')
 
+        self.wrong_syntax("xs:(: invalid QName :)string")
+
     def test_comma_operator(self):
         self.check_value("1, 2", [1, 2])
         self.check_value("(1, 2)", [1, 2])
@@ -262,10 +264,19 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
 
     def test_if_expressions(self):
         root = self.etree.XML('<A><B1><C1/><C2/></B1><B2/><B3><C3/><C4/><C5/></B3></A>')
+
+        token = self.parser.parse("if (1) then 2 else 3")
+        self.assertEqual(len(token), 3)
+        self.assertEqual(token.source, 'if (1) then 2 else 3')
+
         self.check_value("if (1) then 2 else 3", 2)
         self.check_selector("if (true()) then /A/B1 else /A/B2", root, root[:1])
         self.check_selector("if (false()) then /A/B1 else /A/B2", root, root[1:2])
 
+        token = self.parser.parse("if")
+        self.assertEqual(token.symbol, '(name)')
+        self.assertEqual(token.value, 'if')
+
         # Cases from XPath 2.0 examples
         root = self.etree.XML('<part discounted="false"><wholesale/><retail/></part>')
         self.check_selector(
@@ -316,6 +327,10 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value('some $x in (1, 2, "cat") satisfies $x * 2 = 4', True, context)
         self.check_value('every $x in (1, 2, "cat") satisfies $x * 2 = 4', False, context)
 
+        token = self.parser.parse("some")
+        self.assertEqual(token.symbol, '(name)')
+        self.assertEqual(token.value, 'some')
+
         # From W3C XQuery/XPath tests
         context = XPathContext(root=self.etree.XML('<dummy/>'),
                                variables={'result': [43, 44, 45]})
@@ -941,8 +956,7 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_value("() instance of empty-sequence()", True)
 
         self.wrong_syntax("5 instance of unknown()", 'XPST0003', "unknown function 'unknown'")
-        self.wrong_syntax("1e3 instance of empty-sequence()(",
-                          'XPST0003', "unexpected '(' operator")
+        self.wrong_syntax("1e3 instance of empty-sequence()(", 'XPST0003')
 
         # Test dynamic evaluation error on prefixed name
         parser = XPath2Parser()
@@ -1106,7 +1120,7 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         self.check_tokenizer("/is-a", ['/', 'is-a'])
         self.check_tokenizer("/-is-a", ['/', '-', 'is-a'])
 
-    def test_token_ambiguity(self):
+    def test_operator_ambiguity(self):
         # Related to issue #27
         self.check_tokenizer("/is", ['/', 'is'])
         context = XPathContext(self.etree.XML('<root/>'))
@@ -1114,6 +1128,21 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         context = XPathContext(self.etree.XML('<is/>'))
         self.check_value('/is', [context.root], context)
 
+        self.check_value('/and', [], context)
+        context = XPathContext(self.etree.XML('<and/>'))
+        self.check_value('/and', [context.root], context)
+
+        root = self.etree.XML('<and/>')
+        self.check_value('and', [root], XPathContext(self.etree.ElementTree(root)))
+        root = self.etree.XML('<eq/>')
+        self.check_value('eq', [root], XPathContext(self.etree.ElementTree(root)))
+        root = self.etree.XML('<union/>')
+        self.check_value('union', [root], XPathContext(self.etree.ElementTree(root)))
+
+    def test_statements_ambiguity(self):
+        root = self.etree.XML('<for/>')
+        self.check_value('for', [root], XPathContext(self.etree.ElementTree(root)))
+
     @unittest.skipIf(xmlschema is None, "xmlschema library required.")
     def test_get_atomic_value(self):
         schema = xmlschema.XMLSchema(dedent("""\
@@ -1274,6 +1303,22 @@ class XPath2ParserTest(test_xpath1_parser.XPath1ParserTest):
         parser = self.parser.__class__(default_collation='it_IT.UTF-8')
         self.assertEqual(parser.default_collation, 'it_IT.UTF-8')
 
+    def test_issue_35_getting_attribute_names(self):
+        root = self.etree.XML(dedent("""\
+            <!-- <?xml version="1.0" encoding="utf-8"?> --> 
+            <library attrib1="att11" attrib2="att22"> some text 
+              <book isbn="1111111111"> 
+                <title lang="en">T1 T1 T1 T1 T1</title> 
+              </book> 
+              <book isbn="2222222222"> 
+                <title lang="en">T2 T2 T2 T2 T2</title> 
+              </book> 
+            </library>"""))
+
+        result = ['attrib1', 'attrib2', 'isbn', 'lang', 'isbn', 'lang']
+        self.check_selector('//@*/local-name()', root, result)
+        self.check_selector('//@*/name()', root, result)
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath2ParserTest(XPath2ParserTest):
